const express = require('express');

const simulationApp = express();

const libs = `${process.cwd()}/libs`;


const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const Simulation = require(`${libs}/models/simulation`);

simulationApp.get('/', (req, res) => {
    res.send({ success: true, msg: 'controller working' });
});



simulationApp.post('/', (req, res, next) => {
    // This method must associate new entry with user.
    /* Creates a new simulation. Requires a name. */
    if (!req.body.name) {
        res.send({ success: false, msg: 'Must send a name for new entry' });
    } else {
        next();
    }
}, (req, res) => {
    let newSimulation = new Simulation({
        name: req.body.name,
    });
    newSimulation.save((err) => {
        if (err) {
            res.send({ success: false, msg: err });
        } else {
            res.send({
                success: true,
                msg: 'new sim created',
                id: newSimulation._id,
            });
        }
    });
});

simulationApp.get('/:id', (req, res) => {
    /* Can be used to check simulation construction status */
    Simulation.findById(req.params.id, (err, simulation) => {
        log.debug(req.params.id);
        if (err) {
            res.send({ success: false, msg: err });
            return;
        }

        if (!simulation) {
            res.send({ success: false, msg: 'Entry not found' });
        } else {
            res.send(simulation);
        }
    });
});

simulationApp.post('/:id', (req, res, next) => {
    if (!Object.keys(req.body).length) {
        res.send({ success: false, msg: 'No field informed to update' });
    } else {
        next();
    }
}, (req, res, next) => {
    let simulation = Simulation.findById(req.params.id, (err, simulation) => {
        if (err) {
            res.send({ success: false, msg: err });
        } else {
            if (!simulation) {
                res.send({ success: false, msg: 'Entry not found' });
            } else {
                req.simulation = simulation;
                next();
            }
        }
    });
}, (req, res) => {
    for (let property in req.body) {
        if (Simulation.schema.tree.hasOwnProperty(property)) {
            if(!req.simulation.update(property, req.body[property])){
                res.send({
                    success: false,
                    msg: 'Invalid format for ' + property,
                });
                return ;
            }
        } else {
            res.send({
                success: false,
                msg: 'Unknown property ' + property,
            });
            return ;
        }
    }
    req.simulation.save((err) => {
        if (err) {
            res.send({ success: false, msg: err });
        } else {
            res.send({
                success: true,
                msg: 'sim updated',
                id: req.simulation._id,
            });
        }
    });
});

module.exports = simulationApp;
